import os

from django.contrib.auth.tokens import PasswordResetTokenGenerator
from django.http import HttpResponsePermanentRedirect
from django.utils.encoding import smart_str, DjangoUnicodeDecodeError
from django.utils.http import urlsafe_base64_decode

from rest_framework import generics, status
from rest_framework.response import Response

from django_resaas.models.user import User
from django_resaas.core.utils.translate import Translate


class CustomRedirect(HttpResponsePermanentRedirect):
    allowed_schemes = [os.environ.get('APP_SCHEME'), 'http', 'https']


class PasswordTokenCheckAPIView(generics.GenericAPIView):

    def get(self, request, uidb64, token):
        redirect_url = request.GET.get('redirect_url')

        try:
            user_id = smart_str(urlsafe_base64_decode(uidb64))
            user = User.objects.get(id=user_id)

            if not PasswordResetTokenGenerator().check_token(user, token):
                if redirect_url and len(redirect_url) > 3:
                    return CustomRedirect(
                        f'{redirect_url}?token_valid=False'
                    )
                return CustomRedirect(
                    f"{os.environ.get('FRONTEND_URL', '')}?token_valid=False"
                )

            if redirect_url and len(redirect_url) > 3:
                return CustomRedirect(
                    f'{redirect_url}?token_valid=True'
                    f'&message=Credenciais válidas'
                    f'&uidb64={uidb64}&token={token}'
                )

            return CustomRedirect(
                f"{os.environ.get('FRONTEND_URL', '')}?token_valid=False"
            )

        except DjangoUnicodeDecodeError:
            return Response(
                {
                    'alert_error': Translate.tdc(
                        request,
                        'O token não é válido, solicite um novo'
                    )
                },
                status=status.HTTP_400_BAD_REQUEST
            )



# Generated by Django 5.2.10 on 2026-03-13 17:51

import django.db.models.deletion
import uuid
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("django_resaas", "0005_corsallowedorigin"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="theme",
            options={
                "permissions": (),
                "verbose_name": "Theme",
                "verbose_name_plural": "Themes",
            },
        ),
        migrations.AddField(
            model_name="layoutsetting",
            name="border_radius",
            field=models.IntegerField(default=8),
        ),
        migrations.AddField(
            model_name="layoutsetting",
            name="buttonDense",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="layoutsetting",
            name="buttonRounded",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="layoutsetting",
            name="buttonSquare",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="layoutsetting",
            name="inputDense",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="layoutsetting",
            name="inputRounded",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="layoutsetting",
            name="inputSquare",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="theme",
            name="background",
            field=models.CharField(default="#F6F8F7", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="background_dark",
            field=models.CharField(default="#121212", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="border",
            field=models.CharField(default="#E0E0E0", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="button_outline",
            field=models.CharField(default="#1976D2", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="button_primary",
            field=models.CharField(default="#1976D2", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="button_primary_text",
            field=models.CharField(default="#FFFFFF", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="button_secondary",
            field=models.CharField(default="#26A69A", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="button_secondary_text",
            field=models.CharField(default="#FFFFFF", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="card",
            field=models.CharField(default="#FFFFFF", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="divider",
            field=models.CharField(default="#EEEEEE", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="footer",
            field=models.CharField(default="#1976D2", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="footer_text",
            field=models.CharField(default="#FFFFFF", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="grey",
            field=models.CharField(default="#CDCDCD", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="grey_1",
            field=models.CharField(default="#FAFAFA", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="grey_2",
            field=models.CharField(default="#F5F5F5", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="grey_3",
            field=models.CharField(default="#EEEEEE", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="grey_4",
            field=models.CharField(default="#E0E0E0", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="grey_5",
            field=models.CharField(default="#BDBDBD", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="grey_6",
            field=models.CharField(default="#9E9E9E", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="grey_7",
            field=models.CharField(default="#757575", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="grey_8",
            field=models.CharField(default="#616161", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="grey_9",
            field=models.CharField(default="#424242", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="header",
            field=models.CharField(default="#1976D2", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="header_text",
            field=models.CharField(default="#FFFFFF", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="input_background",
            field=models.CharField(default="#FFFFFF", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="input_border",
            field=models.CharField(default="#CCCCCC", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="input_focus",
            field=models.CharField(default="#1976D2", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="light",
            field=models.CharField(default="#FFFFFF", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="page",
            field=models.CharField(default="#FFFFFF", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="sidebar",
            field=models.CharField(default="#FFFFFF", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="sidebar_active",
            field=models.CharField(default="#1976D2", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="sidebar_active_text",
            field=models.CharField(default="#FFFFFF", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="sidebar_hover",
            field=models.CharField(default="#F5F5F5", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="sidebar_text",
            field=models.CharField(default="#333333", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="text_dark",
            field=models.CharField(default="#1D1D1D", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="text_light",
            field=models.CharField(default="#FFFFFF", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="text_link",
            field=models.CharField(default="#1976D2", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="text_muted",
            field=models.CharField(default="#9E9E9E", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="text_primary",
            field=models.CharField(default="#000000", max_length=50),
        ),
        migrations.AddField(
            model_name="theme",
            name="text_secondary",
            field=models.CharField(default="#666666", max_length=50),
        ),
        migrations.AlterField(
            model_name="theme",
            name="accent",
            field=models.CharField(default="#9C27B0", max_length=50),
        ),
        migrations.AlterField(
            model_name="theme",
            name="dark",
            field=models.CharField(default="#1D1D1D", max_length=50),
        ),
        migrations.AlterField(
            model_name="theme",
            name="primary",
            field=models.CharField(default="#1976D2", max_length=50),
        ),
        migrations.AlterField(
            model_name="theme",
            name="warning",
            field=models.CharField(default="#F2C037", max_length=50),
        ),
        migrations.CreateModel(
            name="AnimationSetting",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("deleted_at", models.DateTimeField(blank=True, null=True)),
                (
                    "estado",
                    models.IntegerField(
                        choices=[(0, "Inativo"), (1, "Ativo")], default=0, null=True
                    ),
                ),
                ("nome", models.CharField(max_length=100)),
                ("enable_animations", models.BooleanField(default=True)),
                (
                    "animation_speed",
                    models.CharField(
                        choices=[
                            ("slow", "Slow"),
                            ("normal", "Normal"),
                            ("fast", "Fast"),
                        ],
                        default="normal",
                        max_length=20,
                    ),
                ),
                (
                    "page_transition",
                    models.CharField(
                        choices=[
                            ("none", "None"),
                            ("fade", "Fade"),
                            ("slide-left", "Slide Left"),
                            ("slide-right", "Slide Right"),
                            ("slide-up", "Slide Up"),
                            ("slide-down", "Slide Down"),
                            ("scale", "Scale"),
                            ("zoom", "Zoom"),
                        ],
                        default="fade",
                        max_length=50,
                    ),
                ),
                (
                    "button_animation",
                    models.CharField(
                        choices=[
                            ("none", "None"),
                            ("ripple", "Ripple"),
                            ("pulse", "Pulse"),
                            ("bounce", "Bounce"),
                            ("scale", "Scale"),
                        ],
                        default="ripple",
                        max_length=50,
                    ),
                ),
                ("hover_effect", models.BooleanField(default=True)),
                (
                    "hover_style",
                    models.CharField(
                        choices=[
                            ("none", "None"),
                            ("lift", "Lift"),
                            ("shadow", "Shadow"),
                            ("grow", "Grow"),
                            ("glow", "Glow"),
                        ],
                        default="lift",
                        max_length=50,
                    ),
                ),
                (
                    "card_animation",
                    models.CharField(
                        choices=[
                            ("none", "None"),
                            ("fade", "Fade"),
                            ("slide-up", "Slide Up"),
                            ("zoom", "Zoom"),
                        ],
                        default="fade",
                        max_length=50,
                    ),
                ),
                (
                    "modal_animation",
                    models.CharField(
                        choices=[
                            ("none", "None"),
                            ("scale", "Scale"),
                            ("fade", "Fade"),
                            ("slide-up", "Slide Up"),
                        ],
                        default="scale",
                        max_length=50,
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="%(class)s_created",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "updated_by",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="%(class)s_updated",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "Animation Setting",
                "verbose_name_plural": "Animation Settings",
            },
        ),
        migrations.AddField(
            model_name="entidade",
            name="animation_settings",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="django_resaas.animationsetting",
            ),
        ),
        migrations.AddField(
            model_name="tipoentidade",
            name="animation_settings",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="django_resaas.animationsetting",
            ),
        ),
        migrations.CreateModel(
            name="Typography",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("deleted_at", models.DateTimeField(blank=True, null=True)),
                (
                    "estado",
                    models.IntegerField(
                        choices=[(0, "Inativo"), (1, "Ativo")], default=0, null=True
                    ),
                ),
                ("nome", models.CharField(max_length=100)),
                (
                    "font_family",
                    models.CharField(
                        choices=[
                            ("Roboto", "Roboto"),
                            ("Inter", "Inter"),
                            ("Open Sans", "Open Sans"),
                            ("Lato", "Lato"),
                            ("Poppins", "Poppins"),
                            ("Montserrat", "Montserrat"),
                            ("Source Sans Pro", "Source Sans Pro"),
                            ("Nunito", "Nunito"),
                            ("Raleway", "Raleway"),
                            ("Ubuntu", "Ubuntu"),
                            ("Playfair Display", "Playfair Display"),
                            ("Merriweather", "Merriweather"),
                        ],
                        default="Roboto",
                        max_length=100,
                    ),
                ),
                (
                    "font_family_monospace",
                    models.CharField(
                        choices=[
                            ("monospace", "Default Monospace"),
                            ("Fira Code", "Fira Code"),
                            ("JetBrains Mono", "JetBrains Mono"),
                            ("Source Code Pro", "Source Code Pro"),
                            ("Roboto Mono", "Roboto Mono"),
                        ],
                        default="monospace",
                        max_length=100,
                    ),
                ),
                ("font_size_base", models.IntegerField(default=14)),
                ("font_size_h1", models.IntegerField(default=32)),
                ("font_size_h2", models.IntegerField(default=28)),
                ("font_size_h3", models.IntegerField(default=24)),
                ("font_size_h4", models.IntegerField(default=20)),
                ("font_size_h5", models.IntegerField(default=18)),
                ("font_size_h6", models.IntegerField(default=16)),
                ("font_size_body", models.IntegerField(default=14)),
                ("font_size_caption", models.IntegerField(default=12)),
                ("font_size_small", models.IntegerField(default=11)),
                ("font_weight_light", models.IntegerField(default=300)),
                ("font_weight_normal", models.IntegerField(default=400)),
                ("font_weight_medium", models.IntegerField(default=500)),
                ("font_weight_bold", models.IntegerField(default=700)),
                ("line_height", models.FloatField(default=1.5)),
                ("letter_spacing", models.FloatField(default=0)),
                ("uppercase_headings", models.BooleanField(default=False)),
                (
                    "created_by",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="%(class)s_created",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "updated_by",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="%(class)s_updated",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "Typography",
                "verbose_name_plural": "Typography",
            },
        ),
        migrations.AddField(
            model_name="entidade",
            name="typography",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="django_resaas.typography",
            ),
        ),
        migrations.AddField(
            model_name="tipoentidade",
            name="typography",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="django_resaas.typography",
            ),
        ),
    ]



from django.contrib import admin

@admin.action(description="Restaurar selecionados")
def restore_selected(modeladmin, request, queryset):
    queryset.restore()

@admin.action(description="Soft delete selecionados")
def soft_delete_selected(modeladmin, request, queryset):
    queryset.soft_delete()

class BaseAdmin(admin.ModelAdmin):
    actions = [restore_selected, soft_delete_selected]

    def get_queryset(self, request):
        return self.model.all_objects.all()



from django.db import models
from django_resaas.core.base.models import TimeModel
class TipoEntidadeModulo(TimeModel):
    tipo_entidade = models.ForeignKey("django_resaas.TipoEntidade", on_delete=models.CASCADE)
    modulo = models.ForeignKey("django_resaas.Modulo", on_delete=models.CASCADE)

    class Meta:
        permissions = (
            
        )

    def __str__(self):
        return str(self.tipo_entidade.nome) + "  |  " + str(self.modulo.nome)




import importlib
import importlib.util

from django.apps import apps
from django.conf import settings as dj_settings
from django.contrib.auth.models import Group
from django.contrib.contenttypes.models import ContentType

from rest_framework import viewsets, filters, status
from rest_framework.decorators import action
from rest_framework.response import Response

from django_resaas.core.utils.translate import Translate
from django_resaas.core.utils.full_path import FullPath

from django_resaas.models.tipo_entidade import TipoEntidade
from django_resaas.models.entidade import Entidade
from django_resaas.models.entidade_user import EntidadeUser
from django_resaas.models.tipo_entidade_modulo import TipoEntidadeModulo
from django_resaas.models.sucursal_user_group import SucursalUserGroup
from django_resaas.models.tipo_entidade_modelo import TipoEntidadeModelo
from django_resaas.models.entidade_modelo import EntidadeModelo
from django_resaas.models.theme import Theme, Typography
from django_resaas.models.layout_setting import LayoutSetting, AnimationSetting
from django_resaas.data.theme.serializers.theme import ThemeSerializer, TypographySerializer
from django_resaas.data.layout_setting.serializers.layout_setting import LayoutSettingSerializer, AnimationSettingSerializer


from django_resaas.data.tipo_entidade.serializers.tipo_entidade import (
    TipoEntidadeSerializer
)


class TipoEntidadeAPIView(viewsets.ModelViewSet):
    search_fields = ['id', 'nome']
    filter_backends = (filters.SearchFilter,)

    serializer_class = TipoEntidadeSerializer
    queryset = TipoEntidade.objects.all()
    lookup_field = 'id'

    def get_queryset(self):
        if self.request.query_params.get('all'):
            return self.queryset.order_by('ordem')

        self._paginator = None
        return self.queryset.filter(
            estado=1
        ).order_by('ordem')

    @action(detail=True, methods=['GET'])
    def user_entidades(self, request, id):
        entidades = Entidade.objects.filter(
            tipo_entidade__id=id
        )
        resultado = []

        for entidade in entidades:
            try:
                EntidadeUser.objects.get(
                    entidade=entidade,
                    user=request.user
                )
                logo = FullPath.url(request, entidade.logo.name, temporary=False)
                resultado.append(
                    {
                        'id': entidade.id,
                        'nome': entidade.nome,
                        'logo': logo,
                    }
                )
            except EntidadeUser.DoesNotExist:
                continue

        return Response(resultado, status=status.HTTP_200_OK)

    @action(detail=True, methods=['GET'])
    def entidades(self, request, id):
        entidades = Entidade.objects.filter(
            tipo_entidade__id=id
        )
        resultado = [
            {'id': entidade.id, 'nome': entidade.nome}
            for entidade in entidades
        ]
        return Response(resultado, status=status.HTTP_200_OK)

    @action(detail=True, methods=['PUT'])
    def perfilPut(self, request, id):
        group = Group.objects.get(id=request.data['id'])
        group.name = request.data['name']
        group.save()

        return Response(
            {
                'id': group.id,
                'name': group.name,
                'alert_success': Translate.tdc(
                    request,
                    f'Perfil <b>{group.name}</b> actualizado com sucesso'
                ),
            },
            status=status.HTTP_201_CREATED,
        )

    @action(detail=True, methods=['POST'])
    def perfilPost(self, request, id):
        tipo_entidade = TipoEntidade.objects.get(id=id)
        group = Group.objects.create(name=request.data['name'])

        tipo_entidade.groups.add(group)
        for entidade in Entidade.objects.filter(tipo_entidade_id=id):
            entidade.groups.add(group)

        return Response(
            {
                'id': group.id,
                'name': group.name,
                'alert_success': Translate.tdc(
                    request,
                    f'Perfil <b>{group.name}</b> criado com sucesso'
                ),
            },
            status=status.HTTP_201_CREATED,
        )

    @action(detail=True, methods=['GET'])
    def apps(self, request, id):
        resultado = []

        for app in apps.get_app_configs():
            resultado.append(
                {
                    'name': app.name,
                    'label': app.label,
                    'verbose': app.verbose_name,
                }
            )

        for app in dj_settings.INSTALLED_APPS:
            resultado.append(app)

        return Response(resultado, status=status.HTTP_200_OK)

    @action(detail=True, methods=['GET'])
    def modelos(self, request, id):
        tipo_entidade = TipoEntidade.objects.get(id=id)
        modelos = [
            {
                'id': tem.modelo.id,
                'model': tem.modelo.model,
                'app_label': tem.modelo.app_label,
            }
            for tem in TipoEntidadeModelo.filter(tipo_entidade__id=tipo_entidade.id)
        ]
        return Response(modelos, status=status.HTTP_200_OK)

    @action(detail=True, methods=['POST'])
    def addModelo(self, request, id):
        tipo_entidade = TipoEntidade.objects.get(id=id)
        modelo = ContentType.objects.get(id=request.data['id'])

        TipoEntidadeModelo.objects.get_or_create(tipo_entidade__id=tipo_entidade.id, modelo=modelo)
        for entidade in Entidade.objects.filter(tipo_entidade_id=id):
            EntidadeModelo.objects.get_or_create(entidade__id=entidade.id, modelo=modelo)

        return Response(
            {
                'id': modelo.id,
                'model': modelo.model,
                'alert_success': Translate.tdc(
                    request,
                    f'Aplicação <b>{modelo.model}</b> criada com sucesso'
                ),
            },
            status=status.HTTP_201_CREATED,
        )

    @action(detail=True, methods=['POST'])
    def removeModelo(self, request, id):
        tipo_entidade = TipoEntidade.objects.get(id=id)
        modelo = ContentType.objects.get(id=request.data['id'])

        TipoEntidadeModelo.objects.filter(tipo_entidade__id=tipo_entidade.id, modelo=modelo).delete()
        for entidade in Entidade.objects.filter(tipo_entidade_id=id):
            EntidadeModelo.objects.filter(entidade__id=entidade.id, modelo=modelo).delete()

        return Response(
            {
                'id': modelo.id,
                'model': modelo.model,
                'alert_success': Translate.tdc(
                    request,
                    f'Aplicação <b>{modelo.model}</b> removida com sucesso'
                ),
            },
            status=status.HTTP_201_CREATED,
        )

    @action(detail=True, methods=['GET'])
    def modulos(self, request, *args, **kwargs):
        tipo_entidade = self.get_object()
        relacoes = TipoEntidadeModulo.objects.filter(
            tipo_entidade=tipo_entidade.id
        )

        modulos = [
            {
                'id': rel.modelo.id,
                'nome': rel.modelo.nome,
            }
            for rel in relacoes
        ]

        return Response(modulos, status=status.HTTP_200_OK)


    @action(detail=True, methods=['GET'])
    def themeGet(self, request, *args, **kwargs):
        tipoentidade = self.get_object()
        tipoentidade = TipoEntidade.objects.get(id=tipoentidade.id )
        if tipoentidade.theme:
            theme = ThemeSerializer(Theme.objects.get(id=tipoentidade.theme.id)).data
        else:
            theme = {}
        return Response(theme, status=status.HTTP_200_OK)

    @action(detail=True, methods=['GET'])
    def layoutSettingsGet(self, request, *args, **kwargs):
        tipoentidade = self.get_object()
        tipoentidade = TipoEntidade.objects.get(id=tipoentidade.id )
        if tipoentidade.layout_settings:
            ls = LayoutSettingSerializer(LayoutSetting.objects.get(id=tipoentidade.layout_settings.id)).data
        else:
            ls = {}
        return Response(ls, status=status.HTTP_200_OK)

    
    @action(detail=True, methods=['GET'])
    def typographyGet(self, request, *args, **kwargs):
        tipoentidade = self.get_object()
        tipoentidade = TipoEntidade.objects.get(id=tipoentidade.id )
        if tipoentidade.typography:
            typography = TypographySerializer(Typography.objects.get(id=tipoentidade.typography.id)).data
        else:
            typography = {}
        return Response(typography, status=status.HTTP_200_OK)

    @action(detail=True, methods=['GET'])
    def animationSettingsGet(self, request, *args, **kwargs):
        tipoentidade = self.get_object()
        tipoentidade = TipoEntidade.objects.get(id=tipoentidade.id )
        if tipoentidade.animation_settings:
            animation_settings = AnimationSettingSerializer(AnimationSetting.objects.get(id=tipoentidade.animation_settings.id)).data
        else:
            animation_settings = {}
        return Response(animation_settings, status=status.HTTP_200_OK)



    @action(detail=True, methods=['PUT'])
    def themePut(self, request, *args, **kwargs):
        tipoentidade = self.get_object()

        theme = tipoentidade.theme
        data = request.data

        for key, value in data.items():
            if hasattr(theme, key):
                if key == "created_by" or key == "updated_by":
                    theme.created_by = request.user
                    theme.updated_by = request.user
                else:
                    setattr(theme, key, value)

        theme.save()
        theme = ThemeSerializer(theme).data
        return Response(theme, status=status.HTTP_200_OK)


    @action(detail=True, methods=['PUT'])
    def layoutSettingsPut(self, request, *args, **kwargs):
        tipoentidade = self.get_object()

        layout_settings = tipoentidade.layout_settings
        data = request.data

        for key, value in data.items():
            if hasattr(layout_settings, key):
                if key == "created_by" or key == "updated_by":
                    layout_settings.created_by = request.user
                    layout_settings.updated_by = request.user
                else:
                    setattr(layout_settings, key, value)

        layout_settings.save()
        layout_settings = LayoutSettingSerializer(layout_settings).data
        return Response(layout_settings, status=status.HTTP_200_OK)




    @action(detail=True, methods=['PUT'])
    def typographyPut(self, request, *args, **kwargs):
        tipoentidade = self.get_object()

        typography = tipoentidade.typography
        data = request.data

        for key, value in data.items():
            if hasattr(typography, key):
                if key == "created_by" or key == "updated_by":
                    typography.created_by = request.user
                    typography.updated_by = request.user
                else:
                    setattr(typography, key, value)
        typography.save()
        typography = TypographySerializer(typography).data
        return Response(typography, status=status.HTTP_200_OK)


    @action(detail=True, methods=['PUT'])
    def animationSettingsPut(self, request, *args, **kwargs):
        tipoentidade = self.get_object()

        animation_settings = tipoentidade.animation_settings
        data = request.data

        for key, value in data.items():
            if hasattr(animation_settings, key):
                setattr(animation_settings, key, value)
                if key == "created_by" or key == "updated_by":
                    animation_settings.created_by = request.user
                    animation_settings.updated_by = request.user
                else:
                    setattr(animation_settings, key, value)
        

        animation_settings.save()
        animation_settings = AnimationSettingSerializer(animation_settings).data
        return Response(animation_settings, status=status.HTTP_200_OK)





# 📦 Standard library
import logging
import shutil
from pathlib import Path
from typing import Any, Dict, List, Optional, Tuple

# 📦 Django
from django.apps import apps, apps as django_apps
from django.conf import settings
from django.core.exceptions import PermissionDenied, FieldDoesNotExist
from django.db import models
from django.db.models import Q
from django.http import JsonResponse, Http404

# 📦 Django REST Framework
from rest_framework.views import APIView
from rest_framework.viewsets import ModelViewSet
from rest_framework.permissions import IsAdminUser, IsAuthenticated
from rest_framework.response import Response
from rest_framework.decorators import action

# 📦 Local (django_resaas)
from django_resaas.core.base.views import register_view
from django_resaas.core.base.permissions import hasPermission
from django_resaas.core.utils import ok, fail, warn, all, clean_name
from django_resaas.models.modelo_extra import ModeloExtra
from django_resaas.management.apicommands.service.modulo_service import ModuloScaffoldService

# 🔧 Logger
logger = logging.getLogger(__name__)

# ==========================================================
# helpers
# ==========================================================

LABEL_KEYS = ("nome", "name", "title", "descricao", "description", "label", "codigo", "code", "numero", "num", "id")


def _normalize_model_name(model: str) -> str:
    """
    Aceita: Entidade | entidade | ENTIDADE
    Tenta resolver para o ModelName real.
    """
    m = (model or "").strip()
    if not m:
        return ""
    # Se já parece CamelCase, mantém
    if m[:1].isupper():
        return m
    # tenta Capitalize
    return m[:1].upper() + m[1:]


def _get_model(module: str, model: str):
    module = (module or "").strip()
    model = _normalize_model_name(model)
    if not module or not model:
        raise Http404("module/model required")

    try:
        return apps.get_model(module, model)
    except Exception:
        # tentativa: procurar por label case-insensitive
        try:
            app_config = apps.get_app_config(module.split(".")[-1])
            for m in app_config.get_models():
                if m.__name__.lower() == model.lower():
                    return m
        except Exception:
            pass
        raise Http404(f"model not found: {module}.{model}")


def _field_type(f: models.Field) -> str:
    """
    Retorna string compatível com teu frontend rawTypes:
    CharField, TextField, ForeignKey, ManyToManyField, MoneyField, etc.
    """
    # djmoney MoneyField costuma ter internal_type MoneyField
    it = None
    try:
        it = f.get_internal_type()
    except Exception:
        it = f.__class__.__name__

    # Alguns campos proxy/relacionais podem retornar tipos internos diferentes
    # mas para teu builder basta mapear pelos nomes do Django.
    if isinstance(f, models.ForeignKey):
        return "ForeignKey"
    if isinstance(f, models.OneToOneField):
        return "OneToOneField"
    if isinstance(f, models.ManyToManyField):
        return "ManyToManyField"

    # fallback para o internal type / classname
    return it or f.__class__.__name__


def _relation_str(f: models.Field) -> Optional[str]:
    """
    relation no formato: "app_label.ModelName"
    """
    if not isinstance(f, (models.ForeignKey, models.OneToOneField, models.ManyToManyField)):
        return None

    rel_model = f.remote_field.model
    # pode ser string "app.Model"
    if isinstance(rel_model, str):
        return rel_model

    try:
        app_label = rel_model._meta.app_label
        model_name = rel_model.__name__
        return f"{app_label}.{model_name}"
    except Exception:
        return None


def _extract_min_max_from_validators(validators) -> Tuple[Optional[float], Optional[float], Optional[int], Optional[int]]:
    """
    Retorna: (min_value, max_value, min_length, max_length_validator)
    Observação: CharField max_length é propriedade do campo, mas às vezes também vem via MaxLengthValidator.
    """
    min_v = None
    max_v = None
    min_len = None
    max_len_v = None

    for v in validators or []:
        cname = v.__class__.__name__
        if cname == "MinValueValidator":
            try:
                min_v = float(getattr(v, "limit_value", None))
            except Exception:
                pass
        elif cname == "MaxValueValidator":
            try:
                max_v = float(getattr(v, "limit_value", None))
            except Exception:
                pass
        elif cname == "MinLengthValidator":
            try:
                min_len = int(getattr(v, "limit_value", None))
            except Exception:
                pass
        elif cname == "MaxLengthValidator":
            try:
                max_len_v = int(getattr(v, "limit_value", None))
            except Exception:
                pass

    return min_v, max_v, min_len, max_len_v


def _schema_fields(Model) -> List[Dict[str, Any]]:
    """
    Constrói fields[] pro teu builder.
    """
    out: List[Dict[str, Any]] = []

    # pega campos concretos + m2m
    for f in Model._meta.get_fields():

        if f.name in []:
            continue
        # ignora relações reversas
        if getattr(f, "auto_created", False) and not getattr(f, "concrete", False):
            continue

        # f pode ser ManyToOneRel etc; queremos só Fields
        if not hasattr(f, "name"):
            continue


        # ignora through automático de m2m reverso
        if getattr(f, "many_to_many", False) and getattr(f, "remote_field", None) and getattr(f.remote_field, "through", None):
            # se for o M2M "real" do Model (concrete), deixa passar
            if not getattr(f, "concrete", False):
                continue

        field_obj = f
        if not isinstance(field_obj, (models.Field, models.ManyToManyField)):
            continue

        ftype = _field_type(field_obj)
        relation = _relation_str(field_obj)

        # required: no teu frontend “required = not blank”
        required = True
        try:
            required = not bool(getattr(field_obj, "blank", False))
        except Exception:
            required = True

        # choices
        choices = None
        try:
            if getattr(field_obj, "choices", None):
                ch = list(field_obj.choices)
                # normaliza para [[value,label], ...]
                choices = [[c[0], str(c[1])] for c in ch]
        except Exception:
            choices = None

        # lengths
        max_length = getattr(field_obj, "max_length", None)

        # validators
        validators = getattr(field_obj, "validators", []) or []
        min_v, max_v, min_len, max_len_validator = _extract_min_max_from_validators(validators)

        # se veio MaxLengthValidator e o campo não tem max_length definido, usamos o do validator
        if max_length is None and max_len_validator is not None:
            max_length = max_len_validator

        payload: Dict[str, Any] = {
            "name": field_obj.name,
            "type": ftype,
            "label": str(getattr(field_obj, "verbose_name", field_obj.name) or field_obj.name).replace("_", " ").title(),
            "verbose_name": str(getattr(field_obj, "verbose_name", "") or ""),
            "help_text": str(getattr(field_obj, "help_text", "") or ""),
            "required": bool(required),
            "choices": choices or [],
            "relation": relation,  # None se não for relacional
            "max_length": max_length,
            "min_length": min_len,
            "min": min_v,
            "max": max_v,
        }

        # limpa keys None pra ficar bonito
        payload = {k: v for k, v in payload.items() if v is not None}

        out.append(payload)

    return out


def _guess_label_value(obj) -> str:
    for k in LABEL_KEYS:
        if hasattr(obj, k):
            v = getattr(obj, k, None)
            if v not in (None, ""):
                return str(v)
    return str(obj)



class ModuloSchemaAPIView(ModelViewSet):

    A = []
    serializer_class = None

    def _ensure_dev(self, request):
        if not settings.DEBUG:
            raise PermissionDenied("module_creation_disabled")

    def list(self, request):
        self._ensure_dev(request)
        result = []

        for app in settings.MY_APPS:

            models = [
                m for m in django_apps.get_models()
                if m._meta.app_label == app
            ]

            result.append({
                "name": app,
                "models": len(models)
            })

        return all(request, apps= result)
        


    @hasPermission("delete_modulo")
    def destroy(self, request, pk=None):
        name = pk
        self._ensure_dev(request)

        module_path = Path(settings.BASE_DIR) / name

        if not module_path.exists():
            return fail(request, "module_not_found")

        if name == "django_resaas":
            return warn(request, "module_protected")

        shutil.rmtree(module_path)

        ModuloScaffoldService._remove_from_settings(name)

        return ok(request, "module_deleted_success")

    def create(self, request):

        self._ensure_dev(request)

        name = request.data.get("name")

        if not name:
            return fail(request, "module_name_required")

        try:
            path = ModuloScaffoldService.create(name)

            return ok(
                request,
                "modulo created success",
                path=path
            )

        except Exception as e:
            return fail(
                request,
                "module_creation_failed \n "+ str(e),
            )



    # ======================================================
    # GET /api/django_resaas/modulo/<module>ac
    # lista modelos
    # ======================================================
    def retrieve(self, request, pk=None):

        module = pk
        models = []

        for model in apps.get_models():

            if model._meta.app_label == module:
                models.append(model.__name__)

        return all(request, models=models)


    # ======================================================
    # GET /api/django_resaas/modulo/<module>/<model>/data/
    # Retorna todos os dados do modelo
    # ======================================================
    
    @action(detail=True, methods=["get"], url_path=r"(?P<model>[^/.]+)/data")
    def model_data(self, request, pk=None, model=None):
        module = pk
        Model = _get_model(module, model)

        qs = Model.objects.all()

        data = list(qs.values()[:50])

        return Response({
            "results": data,
            "count": qs.count()
        })

    # ======================================================
    # GET /api/django_resaas/modulo/<module>/<model>/schema/
    # lista campos detalhados
    # ======================================================
    @action(detail=True, methods=["get"], url_path=r"(?P<model>[^/.]+)/schema")
    def model_schema(self, request, pk=None, model=None):
        module = pk
        Model = _get_model(module, model)
        fields = _schema_fields(Model)

        try:
            Model = apps.get_model(module, model)
        except LookupError:
            return fail(request, "model_not_found")

        actions = []

        for m in ModeloExtra.objects.filter(modelo=model):
            actions.append({
                'icon': m.icon,
                'modelo': m.modelo,
                'method': m.method,
                'permission': m.permission,
                'url': m.url,
                'details': m.details,
            })


        
        
        return all(
            request,
            module=module,
            model=model,
            fields=fields,
            actions = actions
        )


class RelationsAPIView(APIView):
    """
    GET /api/django_resaas/relations/?model=app.Model&search=abc
    -> [{id, label}, ...]
    """
    permission_classes = [IsAuthenticated]

    def get(self, request):
        model_str = (request.query_params.get("model") or "").strip()
        search = (request.query_params.get("search") or "").strip()

        if "." not in model_str:
            return Response({"detail": "model param must be app_label.ModelName"}, status=400)

        app_label, model_name = model_str.split(".", 1)
        Model = _get_model(app_label, model_name)

        qs = Model.objects.all()

        # filtro search (tenta nome/name/title/descricao... se existir)
        if search:
            q = Q()
            for k in LABEL_KEYS:
                if k == "id":
                    continue
                try:
                    Model._meta.get_field(k)
                    q |= Q(**{f"{k}__icontains": search})
                except FieldDoesNotExist:
                    continue
                except Exception:
                    continue
            # se não achou nenhum campo “label”, cai no str(obj) (sem filtro)
            if q:
                qs = qs.filter(q)

        qs = qs.order_by("-id")[:50]

        rows = [{"id": o.pk, "label": _guess_label_value(o)} for o in qs]
        return Response(rows, status=200)






from django.core.management.base import BaseCommand
from django_resaas.core.services.traducao_sinc_service import TraducaoSyncService





class Command(BaseCommand):
    help = "Bootstrap inicial do SaaS"

    def handle(self, *args, **options):
        self.stdout.write(self.style.MIGRATE_HEADING("🚀 Sync Traducao SaaS \n\n"))

        TraducaoSyncService.sync( stdout=self.stdout, style=self.style )

        self.stdout.write(self.style.SUCCESS("\n 🛠 ⚙️ Idiomas Sincronizadas, pronto para uso\n"))



from django_resaas.core.base.serializers import BaseSerializer
from django_resaas.models.theme import Theme, Typography


class ThemeSerializer(BaseSerializer):
    permanent_fields_files = []
    class Meta:
        model = Theme
        fields = "__all__"

class TypographySerializer(BaseSerializer):
    permanent_fields_files = []
    class Meta:
        model = Typography
        fields = "__all__"



from rest_framework import status, generics
from rest_framework.response import Response

from django_resaas.core.utils.translate import Translate
from django_resaas.data.user.serializers.me import MeSerializer


class MeAPIView(generics.GenericAPIView):
    serializer_class = MeSerializer

    def get(self, request):
        serializer = self.serializer_class(
            request.user,
            context={'request': request}
        )
        data = serializer.data.copy()
        return Response( data, status=status.HTTP_200_OK)



from django.utils import timezone
from django.shortcuts import get_object_or_404

from django.db.models import Q
from django.core.exceptions import FieldDoesNotExist

from rest_framework.viewsets import ModelViewSet
from rest_framework.exceptions import PermissionDenied

from django_resaas.core.base.permissions import isPermited
from django_resaas.core.utils.translate import Translate
from rest_framework.filters import SearchFilter, OrderingFilter
from django_filters.rest_framework import DjangoFilterBackend

from rest_framework import status
from rest_framework.decorators import action
from rest_framework.response import Response

from django_resaas.core.base.permissions import hasPermission
from django_resaas.core.utils import all, ok, fail, warn  # noqa



from django_resaas.core.base.registry import VIEW_REGISTRY

from django.db.models import Q, ForeignKey, OneToOneField
from django.core.exceptions import FieldDoesNotExist


def build_search_query(Model, search, depth=1):
    """
    depth = nível de profundidade (1 = FK direto)
    """
    q = Q()

    # campos locais
    candidates = [
        "nome", "name", "title", "descricao", "description",
        "username", "email", "codigo", "code"
    ]

    for field in candidates:
        try:
            Model._meta.get_field(field)
            q |= Q(**{f"{field}__icontains": search})
        except FieldDoesNotExist:
            continue

    # 🔥 relações (FK)
    if depth > 0:
        for f in Model._meta.get_fields():
            if isinstance(f, (ForeignKey, OneToOneField)):
                rel_model = f.related_model

                for field in candidates:
                    try:
                        rel_model._meta.get_field(field)
                        q |= Q(**{f"{f.name}__{field}__icontains": search})
                    except FieldDoesNotExist:
                        continue

    return q

def register_view(name=None, module=None):
    def decorator(cls):
        key = name or cls.__name__.lower().replace('APIView', '')+'s'
        module_name = module or cls.__module__.split(".")[0]  # rh, srm, etc

        VIEW_REGISTRY.setdefault(module_name, {})[key] = cls
        return cls
    return decorator


class BaseAPIView(ModelViewSet):
    """
    ViewSet base multi-tenant com controlo automático de permissões.
    """
    filter_backends = [DjangoFilterBackend, SearchFilter, OrderingFilter]
    filterset_fields = "__all__"   # ou lista controlada

    # search_fields = ["id", "name"]
    ordering_fields = "__all__"

    permission_action_map = {
        'list': 'list',
        'retrieve': 'view',
        'create': 'add',
        'update': 'change',
        'partial_update': 'change',
        'destroy': 'delete',
        'restore': 'restore',
        'hard_delete': 'hard_delete',
    }


    def apply_dynamic_search(self, qs):
        search = (self.request.query_params.get("search") or "").strip()

        if not search:
            return qs

        Model = qs.model

        q = build_search_query(Model, search, depth=1)

        return qs.filter(q)
    
    def get_model(self):
        return self.get_queryset().model

    def get_method_permission(self):
        custom_map = getattr(self, 'method_permission', {})
        return {**self.permission_action_map, **custom_map}

    def initial(self, request, *args, **kwargs):
        super().initial(request, *args, **kwargs)

        action = self.action
        model = self.get_queryset().model

        perm_map = self.get_method_permission()
        perm_prefix = perm_map.get(action)

        if not perm_prefix:
            raise PermissionDenied(
                Translate.tdc(request, 'Permissão não definida para esta ação')
            )

        codename = f'{perm_prefix}_{model._meta.model_name}'


        if not isPermited(request=request, role=codename):
            raise PermissionDenied(
                Translate.tdc(request, 'Não autorizado ' +  codename)
            )


    def get_queryset(self):
        qs = super().get_queryset().filter(
            entidade_id=self.request.entidade_id,
            sucursal_id=self.request.sucursal_id,
        )

        status = (self.request.query_params.get("objects") or "").strip()
        
        # Se quiser ver apagados: ?objects=all
        if status == "all":
            Model = qs.model
            if hasattr(Model, "all_objects"):
                return Model.all_objects.filter(
                    entidade_id=self.request.entidade_id,
                    sucursal_id=self.request.sucursal_id,
                )

        # Se quiser so apagados: ?objects=deleted
        if status == "deleted":
            Model = qs.model
            if hasattr(Model, "deleted_objects"):
                return Model.deleted_objects.filter(
                    entidade_id=self.request.entidade_id,
                    sucursal_id=self.request.sucursal_id,
                )

        qs = self.apply_dynamic_search(qs)
        return qs

    def perform_create(self, serializer):
        serializer.save(
            entidade_id=self.request.entidade_id,
            sucursal_id=self.request.sucursal_id,
            created_by=self.request.user,
            updated_by=self.request.user
        )

    def perform_update(self, serializer):
        serializer.save(updated_by=self.request.user)


    def perform_destroy(self, instance):
        if hasattr(instance, "deleted_at"):
            instance.deleted_at = timezone.now()
        instance.delete(user = self.request.user)

    @action(detail=True, methods=["post"], url_path="restore")
    def restore(self, request, pk=None):
        Model = self.get_model()
        instance = get_object_or_404(Model.all_objects, pk=pk)

        if not instance:
            return fail(request, "Objeto não encontrado")

        model = instance._meta.model_name
        codename = f"restore_{model}"

        if not isPermited(request=request, role=codename):
            raise PermissionDenied("Não autorizado")

        instance.restore(user=request.user)  # 🔥 usa teu método do model

        return ok(request, "Restored com sucesso")


    @action(detail=True, methods=["delete"], url_path="hard_delete")
    def hard_delete(self, request, pk=None):
        Model = self.get_model()
        instance = get_object_or_404(Model.all_objects, pk=pk)

        model = instance._meta.model_name
        codename = f"hard_delete_{model}"

        if not isPermited(request=request, role=codename):
            raise PermissionDenied("Não autorizado")

        instance.hard_delete()

        return ok(
            request,
            "Apagado prara sempre com sucesso",
        )

    

    





import uuid
from django.db import models
from django_resaas.core.base.models import TimeModel


class Theme(TimeModel):

    nome = models.CharField(max_length=100)

    # =========================
    # CORE COLORS (QUASAR)
    # =========================

    primary = models.CharField(max_length=50, default="#1976D2")
    secondary = models.CharField(max_length=50, default="#26A69A")
    accent = models.CharField(max_length=50, default="#9C27B0")

    positive = models.CharField(max_length=50, default="#21BA45")
    negative = models.CharField(max_length=50, default="#C10015")
    warning = models.CharField(max_length=50, default="#F2C037")
    info = models.CharField(max_length=50, default="#31CCEC")

    dark = models.CharField(max_length=50, default="#1D1D1D")
    light = models.CharField(max_length=50, default="#FFFFFF")

    # =========================
    # BACKGROUND COLORS
    # =========================

    background = models.CharField(max_length=50, default="#F6F8F7")
    background_dark = models.CharField(max_length=50, default="#121212")

    page = models.CharField(max_length=50, default="#FFFFFF")
    card = models.CharField(max_length=50, default="#FFFFFF")

    # =========================
    # TEXT COLORS
    # =========================

    text_primary = models.CharField(max_length=50, default="#000000")
    text_secondary = models.CharField(max_length=50, default="#666666")
    text_muted = models.CharField(max_length=50, default="#9E9E9E")

    text_light = models.CharField(max_length=50, default="#FFFFFF")
    text_dark = models.CharField(max_length=50, default="#1D1D1D")

    text_link = models.CharField(max_length=50, default="#1976D2")

    # =========================
    # HEADER / FOOTER
    # =========================

    header = models.CharField(max_length=50, default="#1976D2")
    header_text = models.CharField(max_length=50, default="#FFFFFF")

    footer = models.CharField(max_length=50, default="#1976D2")
    footer_text = models.CharField(max_length=50, default="#FFFFFF")

    # =========================
    # SIDEBAR
    # =========================

    sidebar = models.CharField(max_length=50, default="#FFFFFF")
    sidebar_text = models.CharField(max_length=50, default="#333333")

    sidebar_active = models.CharField(max_length=50, default="#1976D2")
    sidebar_active_text = models.CharField(max_length=50, default="#FFFFFF")

    sidebar_hover = models.CharField(max_length=50, default="#F5F5F5")

    # =========================
    # BUTTON COLORS
    # =========================

    button_primary = models.CharField(max_length=50, default="#1976D2")
    button_primary_text = models.CharField(max_length=50, default="#FFFFFF")

    button_secondary = models.CharField(max_length=50, default="#26A69A")
    button_secondary_text = models.CharField(max_length=50, default="#FFFFFF")

    button_outline = models.CharField(max_length=50, default="#1976D2")

    # =========================
    # INPUTS / FORM
    # =========================

    input_background = models.CharField(max_length=50, default="#FFFFFF")
    input_border = models.CharField(max_length=50, default="#CCCCCC")
    input_focus = models.CharField(max_length=50, default="#1976D2")

    # =========================
    # BORDER / DIVIDER
    # =========================

    border = models.CharField(max_length=50, default="#E0E0E0")
    divider = models.CharField(max_length=50, default="#EEEEEE")

    # =========================
    # GREY SCALE
    # =========================

    grey = models.CharField(max_length=50, default="#CDCDCD")
    grey_1 = models.CharField(max_length=50, default="#FAFAFA")
    grey_2 = models.CharField(max_length=50, default="#F5F5F5")
    grey_3 = models.CharField(max_length=50, default="#EEEEEE")
    grey_4 = models.CharField(max_length=50, default="#E0E0E0")
    grey_5 = models.CharField(max_length=50, default="#BDBDBD")
    grey_6 = models.CharField(max_length=50, default="#9E9E9E")
    grey_7 = models.CharField(max_length=50, default="#757575")
    grey_8 = models.CharField(max_length=50, default="#616161")
    grey_9 = models.CharField(max_length=50, default="#424242")

    class Meta:
        verbose_name = 'Theme'
        verbose_name_plural = 'Themes'
        permissions = ()

    def to_dict(self):
        return {

            # CORE COLORS
            "primary": self.primary,
            "secondary": self.secondary,
            "accent": self.accent,
            "positive": self.positive,
            "negative": self.negative,
            "warning": self.warning,
            "info": self.info,
            "dark": self.dark,
            "light": self.light,

            # BACKGROUNDS
            "background": self.background,
            "background_dark": self.background_dark,
            "page": self.page,
            "card": self.card,

            # TEXT
            "text_primary": self.text_primary,
            "text_secondary": self.text_secondary,
            "text_muted": self.text_muted,
            "text_light": self.text_light,
            "text_dark": self.text_dark,
            "text_link": self.text_link,

            # HEADER / FOOTER
            "header": self.header,
            "header_text": self.header_text,
            "footer": self.footer,
            "footer_text": self.footer_text,

            # SIDEBAR
            "sidebar": self.sidebar,
            "sidebar_text": self.sidebar_text,
            "sidebar_active": self.sidebar_active,
            "sidebar_active_text": self.sidebar_active_text,
            "sidebar_hover": self.sidebar_hover,

            # BUTTONS
            "button_primary": self.button_primary,
            "button_primary_text": self.button_primary_text,
            "button_secondary": self.button_secondary,
            "button_secondary_text": self.button_secondary_text,
            "button_outline": self.button_outline,

            # INPUT
            "input_background": self.input_background,
            "input_border": self.input_border,
            "input_focus": self.input_focus,

            # BORDERS
            "border": self.border,
            "divider": self.divider,

            # GREY SCALE
            "grey": self.grey,
            "grey_1": self.grey_1,
            "grey_2": self.grey_2,
            "grey_3": self.grey_3,
            "grey_4": self.grey_4,
            "grey_5": self.grey_5,
            "grey_6": self.grey_6,
            "grey_7": self.grey_7,
            "grey_8": self.grey_8,
            "grey_9": self.grey_9,
        }

    def __str__(self):
        return self.nome





class Typography(TimeModel):
    nome = models.CharField(max_length=100)

    FONT_FAMILY_CHOICES = [
        ("Roboto", "Roboto"),
        ("Inter", "Inter"),
        ("Open Sans", "Open Sans"),
        ("Lato", "Lato"),
        ("Poppins", "Poppins"),
        ("Montserrat", "Montserrat"),
        ("Source Sans Pro", "Source Sans Pro"),
        ("Nunito", "Nunito"),
        ("Raleway", "Raleway"),
        ("Ubuntu", "Ubuntu"),
        ("Playfair Display", "Playfair Display"),
        ("Merriweather", "Merriweather"),
    ]

    MONOSPACE_CHOICES = [
        ("monospace", "Default Monospace"),
        ("Fira Code", "Fira Code"),
        ("JetBrains Mono", "JetBrains Mono"),
        ("Source Code Pro", "Source Code Pro"),
        ("Roboto Mono", "Roboto Mono"),
    ]

    # FONT FAMILY
    font_family = models.CharField(
        max_length=100,
        choices=FONT_FAMILY_CHOICES,
        default="Roboto"
    )

    font_family_monospace = models.CharField(
        max_length=100,
        choices=MONOSPACE_CHOICES,
        default="monospace"
    )

    # BASE SIZE
    font_size_base = models.IntegerField(default=14)

    # HEADINGS
    font_size_h1 = models.IntegerField(default=32)
    font_size_h2 = models.IntegerField(default=28)
    font_size_h3 = models.IntegerField(default=24)
    font_size_h4 = models.IntegerField(default=20)
    font_size_h5 = models.IntegerField(default=18)
    font_size_h6 = models.IntegerField(default=16)

    # BODY
    font_size_body = models.IntegerField(default=14)
    font_size_caption = models.IntegerField(default=12)
    font_size_small = models.IntegerField(default=11)

    # WEIGHTS
    font_weight_light = models.IntegerField(default=300)
    font_weight_normal = models.IntegerField(default=400)
    font_weight_medium = models.IntegerField(default=500)
    font_weight_bold = models.IntegerField(default=700)

    # SPACING
    line_height = models.FloatField(default=1.5)
    letter_spacing = models.FloatField(default=0)

    # TEXT TRANSFORM
    uppercase_headings = models.BooleanField(default=False)

    class Meta:
        verbose_name = "Typography"
        verbose_name_plural = "Typography"

    def to_dict(self):
        return {
            "font_family": self.font_family,
            "font_family_monospace": self.font_family_monospace,

            "font_size_base": self.font_size_base,

            "font_size_h1": self.font_size_h1,
            "font_size_h2": self.font_size_h2,
            "font_size_h3": self.font_size_h3,
            "font_size_h4": self.font_size_h4,
            "font_size_h5": self.font_size_h5,
            "font_size_h6": self.font_size_h6,

            "font_size_body": self.font_size_body,
            "font_size_caption": self.font_size_caption,
            "font_size_small": self.font_size_small,

            "font_weight_light": self.font_weight_light,
            "font_weight_normal": self.font_weight_normal,
            "font_weight_medium": self.font_weight_medium,
            "font_weight_bold": self.font_weight_bold,

            "line_height": self.line_height,
            "letter_spacing": self.letter_spacing,

            "uppercase_headings": self.uppercase_headings
        }

    def __str__(self):
        return self.nome



# const font = typography.font_family

# const link = document.createElement("link")
# link.href = `https://fonts.googleapis.com/css2?family=${font.replace(" ", "+")}:wght@300;400;500;700&display=swap`
# link.rel = "stylesheet"

# document.head.appendChild(link)

# document.body.style.fontFamily = font